__doc__ = """SecClabWeb product module."""
__version__ = '0.1'

from Globals import DTMLFile,InitializeClass,HTML,MessageDialog,HTMLFile,Persistent
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from OFS.Image import File
from App.Common import package_home
from DateTime import DateTime
from OFS.PropertyManager import PropertyManager

from OFS import SimpleItem
import Acquisition                # Various acquisition types. 
from AccessControl.Role import RoleManager         # Zope Security management.

import xml,sys
import string,traceback

PERM_CONTENT = "Access contents information"

manage_addSecClabApiResponseForm = DTMLFile('dtml/manage_addSecClabApiResponseForm', globals())


def manage_addSecClabApiResponse(self, id, title='', REQUEST=None):
    self._setObject(id, SecClabApiResponse(id, title))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
    
class SecClabApiResponse(PropertyManager,
                         Persistent,
                         RoleManager,
                         SimpleItem.Item,
                         Acquisition.Implicit,  # Let us use our parent's variables and methods
                         ):
     meta_type = 'SecClabApiResponse'

     manage=manage_main=DTMLFile('dtml/manageSecClabApiResponse', globals())
     manage_main._setName('manage_main')
     
     _properties=({'id':'title', 'type': 'string','mode':'wd'},)

     manage_options = (
	{'label': 'Edit',       'action': 'manage_main'},
	{'label': 'View',       'action': ''},
        )+ RoleManager.manage_options + SimpleItem.Item.manage_options
     # + SimpleItem.SimpleItem.manage_options + PropertyManager.manage_options
     

     __ac_permissions__=(
	('View management screens', ('manage_tabs', 'manage_main')),
	('Change permissions',      ('manage_access',)           ),
	)

     _security = ClassSecurityInfo()

     # set security for the object itself, e.g. if it is accessed in DTML code
     # This line is REQUIRED to allow access to External Files by the public (ariel DTDs, etc.)
     _security.declareObjectPublic()
     
     def __init__(self, id, title='',function_map={}): 
         self.id = id
         self.title = title
         self._function_map = function_map


     def _getText(nodelist):
         rc = ""
         for node in nodelist:
             if node.nodeType == node.TEXT_NODE:
                 rc = rc + node.data
         return rc
     
     _security.declareProtected(PERM_CONTENT, 'index_html')
     # next line is not strictly necessary, Access contents info is Anonymous by default
     # as opposed to all other protections that get Manager Role by default
     _security.setPermissionDefault(PERM_CONTENT, ('Anonymous')) 
     def index_html(self, REQUEST, RESPONSE):
         """   """
         RESPONSE.setHeader('Cache-Control', 'no-cache')
         RESPONSE.setHeader('Pragma', 'no-cache')
         RESPONSE.setHeader('Expires', '-1')

         # decode de request
         service_name="none"
         method_name="none"
         parsedDoc = None
         
         try:
             if hasattr(REQUEST,'BODY'):
                 f = open("/tmp/log_web-api.txt","a+")
                 f.write("----------------------------\n")
                 f.write(REQUEST['BODY'])
                 f.write("\n----------------------------\n")
                 f.close()
                 try:
                     parsedDoc = xml.dom.minidom.parseString(REQUEST['BODY'])
                     service_name = parsedDoc.firstChild.tagName
                     method_name  = parsedDoc.firstChild.childNodes[0].tagName
                 except:
                     get_transaction().abort()
                     error = "%s" % sys.exc_info()[1]
                     ext_error = "%s\r\n%s\r\n%s" % (sys.exc_info()[0],sys.exc_info()[1],string.join(traceback.format_list(traceback.extract_tb(sys.exc_info()[2])),"\r\n"))
                     self._RaiseError(RESPONSE,service_name,method_name,error,ext_error);
                     return
                     

                 if method_name in self._function_map:
                     method = getattr(self.aq_parent,self._function_map[method_name])
                     method(parsedDoc,REQUEST,RESPONSE)
                     return


             if self._function_map.has_key(''):
                 method = getattr(self.aq_parent,self._function_map[''])
                 method(parsedDoc,REQUEST,RESPONSE)
                 return
             else:
                 raise "ERROR","Method name not found (%s)" % method_name
                 
         except:
             # tell zope undo all operations
             get_transaction().abort()
             error = "%s" % sys.exc_info()[1]
             ext_error = "%s\r\n%s\r\n%s" % (sys.exc_info()[0],sys.exc_info()[1],string.join(traceback.format_list(traceback.extract_tb(sys.exc_info()[2])),"\r\n"))
             # tell zope undo all operations

             f = open("/tmp/log_web-api.txt","a+")
             f.write("----------------------------\n")
             f.write(error)
             f.write(ext_error)
             f.write("\n----------------------------\n")
             f.close()

             self._ConstructResponseError(parsedDoc,error,ext_error,RESPONSE)
             

         return

     def _RaiseError(self,RESPONSE,service_name,method_name,error,ext_error):
         doc = xml.dom.minidom.Document()
         node1 = doc.createElement(service_name)
         node2 = doc.createElement(method_name)
         node3 = doc.createElement("query")
         node2.appendChild(node3)
         
         node3 = doc.createElement("response")
         node2.appendChild(node3)
         node4 = doc.createElement("ExitCode")
         node3.appendChild(node4)
         node4.appendChild(doc.createTextNode("-1"))

         node4 = doc.createElement("ErrorMsg")
         node3.appendChild(node4)
         node4.appendChild(doc.createTextNode(error.encode('utf8')))

         node4 = doc.createElement("ExtendedErrorMsg")
         node3.appendChild(node4)
         node4.appendChild(doc.createTextNode(ext_error.encode('utf8')))

         node1.appendChild(node2)
         doc.appendChild(node1)
         doc.normalize()

         message = doc.toxml().encode('utf-8')
         # message = '<?xml version="1.0"?><root><root><root/></root></root>'
         RESPONSE.setHeader("content-type",'text/xml; charset="utf-8"')
         RESPONSE.setHeader('Content-Length', len(message))
         RESPONSE.write(message)

     def _ConstructResponseError(self,decodedXml,error,ext_error,RESPONSE):
         # tell zope undo all operations
         get_transaction().abort()
         responseNode = decodedXml.getElementsByTagName("response")[-1]
         
         node = decodedXml.createElement("ExitCode")
         node.appendChild(decodedXml.createTextNode("1"))
         responseNode.appendChild(node)
         
         node = decodedXml.createElement("ErrorMsg")
         node.appendChild(decodedXml.createTextNode(error.encode('utf8')))
         responseNode.appendChild(node)

         node = decodedXml.createElement("ExtendedErrorMsg")
         node.appendChild(decodedXml.createTextNode(ext_error.encode('utf8')))
         responseNode.appendChild(node)

         decodedXml.normalize()
         
         message = decodedXml.toxml().encode('utf-8')
         RESPONSE.setHeader("content-type",'text/xml; charset="utf-8"')
         RESPONSE.setHeader('Content-Length', len(message))
         RESPONSE.write(message)
         return
